//+------------------------------------------------------------------+
//|                                          optimal tracking filter |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1  PaleVioletRed
#property indicator_color2  DeepSkyBlue
#property indicator_color3  DeepSkyBlue
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2

//
//
//
//
//

extern string TimeFrame          = "Current time frame";
extern int    Price              = PRICE_MEDIAN;
extern int    PriceToCompareTo   = PRICE_CLOSE;
extern int    Passes             = 1;
extern bool   InheritedHighLow   = false;
extern bool   MultiColor         = true;
extern bool   Interpolate        = true;
extern bool   alertsOn           = false;
extern bool   alertsOnCurrent    = false;
extern bool   alertsMessage      = true;
extern bool   alertsSound        = false;
extern bool   alertsEmail        = false;

//
//
//
//
//

double otf[];
double otfUa[];
double otfUb[];
double trend[];

//
//
//
//
//

int    timeFrame;
string indicatorFileName;
bool   returnBars;
bool   calculateValue;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,otf);
   SetIndexBuffer(1,otfUa);
   SetIndexBuffer(2,otfUb);
   SetIndexBuffer(3,trend);
   
      //
      //
      //
      //
      //
      
         Passes = MathMax(MathMin(Passes,25),1);
         indicatorFileName = WindowExpertName();
         calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
         returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
         timeFrame         = stringToTimeFrame(TimeFrame);

      //
      //
      //
      //
      //
   
      string PriceType;
         switch(Price)
         {
            case PRICE_CLOSE:    PriceType = "Close";    break;  // 0
            case PRICE_OPEN:     PriceType = "Open";     break;  // 1
            case PRICE_HIGH:     PriceType = "High";     break;  // 2
            case PRICE_LOW:      PriceType = "Low";      break;  // 3
            case PRICE_MEDIAN:   PriceType = "Median";   break;  // 4
            case PRICE_TYPICAL:  PriceType = "Typical";  break;  // 5
            case PRICE_WEIGHTED: PriceType = "Weighted"; break;  // 6
         }      

   //
   //
   //
   //
   //

   IndicatorShortName(timeFrameToString(timeFrame)+" Optimal tracking filter ("+PriceType+")");
   return(0);
}
int deinit() { return(0); }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double buff[][100];
#define _prc  0
#define _tmp1 1
#define _tmp2 2
#define _otf  3

//
//
//
//
//

int start()
{
   int i,r,limit,counted_bars=IndicatorCounted();

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { otf[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   //

   if (calculateValue || timeFrame == Period())
   {
      if (ArrayRange(buff,0) != Bars) ArrayResize(buff,Bars);
      if (!calculateValue && MultiColor && trend[limit]==1) CleanPoint(limit,otfUa,otfUb);
      for(i=limit, r=Bars-i-1; i >= 0; i--,r++)
      {
         switch (Price)
         {
            case 7 :  buff[r][_prc] = (Open[i]+Close[i])/2.0;                break;
            case 8 :  buff[r][_prc] = (Open[i]+Close[i]+High[i]+Low[i])/4.0; break;
            default : buff[r][_prc] = iMA(NULL,0,1,0,MODE_SMA,Price,i);
         }
      
         //
         //
         //
         //
         //

            for(int k=0; k<Passes; k++)
            {
               buff[r][_tmp1+k*4] = 0.2*(buff[r][_prc+k*4]-buff[r-1][_prc+k*4])+0.8*buff[r-1][_tmp1+k*4];
               if (InheritedHighLow)
                     if (k==0)
                           buff[r][_tmp2+k*4] = 0.1*(High[i]-Low[i])      +0.8*buff[r-1][_tmp2+k*4];
                     else  buff[r][_tmp2+k*4] = 0.1*buff[r][_tmp2+(k-1)*4]+0.8*buff[r-1][_tmp2+k*4];
               else  buff[r][_tmp2+k*4] = 0.1*(High[i]-Low[i])+0.8*buff[r-1][_tmp2+k*4];
               
               //
               //
               //
               //
               //
               
               if (buff[r][_tmp2+k*4]!=0)
                     double lambda = MathAbs(buff[r][_tmp1+k*4]/buff[r][_tmp2+k*4]);
               else         lambda = 0;
               double alpha = ( -lambda*lambda + MathSqrt(lambda*lambda*lambda*lambda + 16.0*lambda*lambda))/8.0;
                  buff[r][_otf+k*4] = buff[r-1][_otf+k*4]+alpha*(buff[r][_prc+k*4]-buff[r-1][_otf+k*4]);
            
               //
               //
               //
               //
               //
            
               if (k<(Passes-1)) buff[r][_prc+(k+1)*4] = buff[r][_otf+k*4];
            }            
            otf[i] = buff[r][_otf+(Passes-1)*4];

         //
         //
         //
         //
         //

         trend[i] = trend[i+1];
         otfUa[i] = EMPTY_VALUE;
         otfUb[i] = EMPTY_VALUE;

         //
         //
         //
         //
         //
         
         double price;
         switch (PriceToCompareTo)
         {
            case 7 :  price = (Open[i]+Close[i])/2.0;                break;
            case 8 :  price = (Open[i]+Close[i]+High[i]+Low[i])/4.0; break;
            default : price = iMA(NULL,0,1,0,MODE_SMA,PriceToCompareTo,i);
         }
         if (price > otf[i]) trend[i] =  1;
         if (price < otf[i]) trend[i] = -1;
         if (!calculateValue && MultiColor && trend[i]==1) PlotPoint(i,otfUa,otfUb,otf);
      }
      manageAlerts();
      return(0);
   }
     
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   if (MultiColor && trend[limit]==1) CleanPoint(limit,otfUa,otfUb);
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         otfUa[i] = EMPTY_VALUE;
         otfUb[i] = EMPTY_VALUE;
         otf[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Price,PriceToCompareTo,Passes,InheritedHighLow,0,y);
         trend[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Price,PriceToCompareTo,Passes,InheritedHighLow,3,y);
            
         //
         //
         //
         //
         //
      
         if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(    k = 1; k < n; k++)
               otf[i+k] = otf[i] + (otf[i+n]-otf[i])*k/n;
   }
   if (MultiColor) for(i=limit; i>=0; i--) if (trend[i]==1) PlotPoint(i,otfUa,otfUb,otf);

   //
   //
   //
   //
   //
   
   manageAlerts();
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"abowe");
         if (trend[whichBar] ==-1) doAlert(whichBar,"bellow");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   string addition = "";
      if (!alertsOnCurrent) addition="closed ";
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(timeFrameToString(timeFrame)," ",Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," price "+addition+doWhat+" optimal tracking filter");
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"otf "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}